@@ -36,7 +36,9 @@ class TestMaker(object):
     """
 
     def __init__(self, test_base_name, test_defaults, test_directory,
-                 fixture_classes, loader, host, port, intercept, prefix):
+                 fixture_classes, loader, host, port, intercept, prefix,
+                 response_handlers, content_handlers, test_loader_name=None,
+                 inner_fixtures=None):
         self.test_base_name = test_base_name
         self.test_defaults = test_defaults
         self.default_keys = set(test_defaults.keys())
@@ -47,6 +49,10 @@ class TestMaker(object):
         self.loader = loader
         self.intercept = intercept
         self.prefix = prefix
+        self.test_loader_name = test_loader_name
+        self.inner_fixtures = inner_fixtures or []
+        self.content_handlers = content_handlers
+        self.response_handlers = response_handlers
 
     def make_one_test(self, test_dict, prior_test):
         """Create one single HTTPTestCase.
@@ -74,6 +80,10 @@ class TestMaker(object):
 
         http_class = httpclient.get_http(verbose=test['verbose'],
                                          caption=test['name'])
+        if prior_test:
+            history = prior_test.history
+        else:
+            history = {}
 
         # Use metaclasses to build a class of the necessary type
         # and name with relevant arguments.
@@ -81,14 +91,24 @@ class TestMaker(object):
                             {'test_data': test,
                              'test_directory': self.test_directory,
                              'fixtures': self.fixture_classes,
+                             'inner_fixtures': self.inner_fixtures,
                              'http': http_class,
                              'host': self.host,
                              'intercept': self.intercept,
+                             'content_handlers': self.content_handlers,
+                             'response_handlers': self.response_handlers,
                              'port': self.port,
                              'prefix': self.prefix,
-                             'prior': prior_test})
+                             'prior': prior_test,
+                             'history': history,
+                             })
+        # We've been asked to, make this test class think it comes
+        # from a different module.
+        if self.test_loader_name:
+            klass.__module__ = self.test_loader_name
 
         tests = self.loader.loadTestsFromTestCase(klass)
+        history[test['name']] = tests._tests[0]
         # Return the first (and only) test in the klass.
         return tests._tests[0]
 
@@ -157,7 +177,9 @@ class TestBuilder(type):
 
 
 def test_suite_from_dict(loader, test_base_name, suite_dict, test_directory,
-                         host, port, fixture_module, intercept, prefix=''):
+                         host, port, fixture_module, intercept, prefix='',
+                         handlers=None, test_loader_name=None,
+                         inner_fixtures=None):
     """Generate a GabbiSuite from a dict represent a list of tests.
 
     The dict takes the form:
@@ -179,8 +201,19 @@ def test_suite_from_dict(loader, test_base_name, suite_dict, test_directory,
         # the original exception in favor of a generic error
         raise GabbiFormatError('malformed test file, invalid format')
 
+    handlers = handlers or []
+    response_handlers = []
+    content_handlers = []
+
     # Merge global with per-suite defaults
     default_test_dict = copy.deepcopy(case.HTTPTestCase.base_test)
+    for handler in handlers:
+        default_test_dict.update(handler.test_base)
+        if handler.response_handler:
+            response_handlers.append(handler.response_handler)
+        if handler.content_handler:
+            content_handlers.append(handler.content_handler)
+
     local_defaults = _validate_defaults(suite_dict.get('defaults', {}))
     test_update(default_test_dict, local_defaults)
 
@@ -193,7 +226,9 @@ def test_suite_from_dict(loader, test_base_name, suite_dict, test_directory,
 
     test_maker = TestMaker(test_base_name, default_test_dict, test_directory,
                            fixture_classes, loader, host, port, intercept,
-                           prefix)
+                           prefix, response_handlers, content_handlers,
+                           test_loader_name=test_loader_name,
+                           inner_fixtures=inner_fixtures)
     file_suite = suite.GabbiSuite()
     prior_test = None
     for test_dict in test_data:
